(function () {
  function submitCurrentForm(modalEl, form) {
    var successCallback = function (data) {
      function createTeamUpdatedMessage(teamName) {
        return $("<div>")
            .addClass('alert team-success')
            .append($("<i>")
                .addClass("fa fa-check-circle")
            )
            .append($("<strong>").text(teamName))
            .append(" was updated successfully.");
      }

      function createEmailSuccessMessage(userAdded, email) {
        return $("<div>")
            .addClass('alert team-success')
            .append($("<i>")
                .addClass("fa fa-check-circle")
            )
            .append("Email " + (userAdded ? "invite" : "notification") + " was sent to ")
            .append($("<strong>").text(email))
            .append(".");
      }

      function createEmailFailMessage(userAdded, email) {
        return $("<div>")
            .addClass('alert team-error')
            .append('<i class="fa fa-times-circle" aria-hidden="true"></i>')
            .append($("<div>")
                .append("Sending " + (userAdded ? "invite email" : "email notification") + " to ")
                .append($("<strong>").text(email))
                .append(" failed.")
                .append($("<div>")
                    .append("Check your ")
                    .append("<a href='" + LS.routes.configurationSmtp +"'>email sending settings<a/>")
                    .append(".<br/>You can resend the email from ")
                    .append("<a href='" + LS.routes.configurationUsers +"'>Configuration > Users<a/>")
                    .append(".")
                )
            );
      }

      if (data.isCloseImmediately) {
        location.reload();
        return;
      }
      else {
        var body = modalEl.find('.team-result-body');
        body.append(createTeamUpdatedMessage(data.teamName));
        if (data.isEmailSuccess) {
          body.append(createEmailSuccessMessage(data.userAdded, data.email));
        }
        else {
          body.append(createEmailFailMessage(data.userAdded, data.email));
        }
      }
      modalEl.find(".team-form").hide();
      modalEl.find(".team-result").show();
      modalEl.find(".team-verify-email").hide();
      modalEl.on('hidden', function () {
        location.reload();
      });
    };

    $.ajax({
      type: form.attr("method"),
      url: form.data("save-url"),
      data: form.serializeArray(),
      dataType: 'json',
      headers: {
        Accept: 'application/json'
      },
      error: function() {
        location.reload();
      },
      success: successCallback,
      complete: function () {
        form.find("button").removeAttr("disabled");
      }
    });
  }

  $(".team-verify-email-no").click(function() {
    var button = $(this);
    var formId = button.data('form-id');
    var modal = $("#" + formId);
    modal.find(".team-form").show();
    modal.find(".team-verify-email").hide();
  });

  $(".team-verify-email-yes").click(function() {
    var button = $(this);
    var formId = button.data('form-id');
    var modal = $("#" + formId);
    submitCurrentForm(modal, modal.find("form[name='teamForm']"));
  });

  $(".uniquesButtton[data-url]").click(function (e) {
    e.preventDefault();

    var uniqueButton = $(this);

    return $.getJSON(uniqueButton.data("url"), function (resp) {

      var modalEl = $("#editTeamPopupContent");
      var form = modalEl.find("form[name='teamForm']");

      modalEl.find(".team-form").show();
      modalEl.find(".team-verify-email").hide();
      resetErrors(modalEl);
      populateEditModelWithData(modalEl, resp);
      enableButton(form);

      modalEl.on('hide', function(){
        modalEl.removeClass('modal');
      }).modal('show').addClass('modal');

      teamModalAjaxFormSubmit(modalEl);
    });
  });

  function populateEditModelWithData(modalEl, data) {
    modalEl.find("input[name='id']").val(data.id);
    modalEl.find("input[name='name']").val(data.name);
    modalEl.find("input[name='teamLeadEmail']").val(data.teamLeadEmail);
    if (data.subnetMask) {
      modalEl.find("input.restrict-subnet").prop('checked', true);
      modalEl.find("input[name='subnetMask']").val(data.subnetMask).prop('disabled', false);
    }
    else {
      modalEl.find("input.restrict-subnet").prop('checked', false);
      var maskField = modalEl.find("input[name='subnetMask']");
      maskField.prop('disabled', true);
      maskField.val('');
    }

    $.each(data.tokens, function (index, token) {
      var row = modalEl.find("div[data-product='" + token.product + "']");
      row.find("input.team-product-id").val(token.id);
      var checkbox = row.find("input.team-product-checkbox");
      checkbox.prop('checked', token.active);
      checkbox.prop('disabled', token.active);
      var limit = row.find("input.team-product-limit");
      limit.val(token.maxUsers);
      limit.prop('disabled', !token.active);
    });
  }

  //Enable subnet restriction checkbox
  $("#editTeamPopupContent").find("input.restrict-subnet").click(function () {
    var field = $("#editTeamPopupContent").find("input[name=subnetMask]");
    field.prop('disabled', !this.checked);
    field.val('');
  });

  //ESC - Closes without saving
  $(document).keyup(function(e) {
    if (e.keyCode === 27) {
      $('#sendTokenContent').modal('hide');
      $('#editTeamPopupContent').modal('hide');
      $('#addTeamPopupContent').modal('hide');
    }
  });

  /// Invite license users form (emailTemplate.scala.html) ///
  $(".sendToken").click(function () {
    var templateDataUrl = $(this).data('templateDataUrl');
    var groupId = $(this).data("groupId");
    $.getJSON(templateDataUrl.replace('/0', '/' + groupId))
      .success(function (resp) {
        prepareModalWindow(resp);

        var modalEl = $("#sendTokenContent");
        modalEl.on('hide', function () {
          modalEl.removeClass('modal');
        }).modal('show').addClass('modal');

        var form = modalEl.find("form[name='sendEmailTeam']");
        LS.common.ajaxFormSubmit(form, function () {
          modalEl.modal('hide');
        }, function (resp) {
          var errors = _.pairs($.parseJSON(resp.responseText));
          form.find(".alert").html(errors[0][1]).show();
          form.find(".fa-spinner").hide();
        }, function () {
          modalEl.find(".alert").hide();
          disableButton(modalEl.find("form[name='sendEmailTeam']"));
          modalEl.find(".fa-spinner").show();
        });
      });
    return false;
  });

  $(".createNewTeam").click(function () {
    resetAddTeamModal();
    var modalEl = $("#addTeamPopupContent");
    modalEl.on('hide', function(){
      modalEl.removeClass('modal');
    }).modal('show').addClass('modal');

    teamModalAjaxFormSubmit(modalEl);
    return false;
  });

  function teamModalAjaxFormSubmit(modalEl) {
    var form = modalEl.find("form[name='teamForm']");
    LS.common.ajaxFormSubmit(form, function (data) {
      if (data.isNewUser) {
        modalEl
            .find(".team-verify-email-body")
            .empty()
            .append($("<div>")
                .append("You are about to invite a new user ")
                .append($("<strong>").text(data.email))
                .append(" as a team lead for ")
                .append($("<strong>").text(data.teamName))
                .append(".")
            )
            .append($("<div>")
                .append("This user will gain restricted access to this License Server instance.")
            );
        modalEl.find(".team-form").hide();
        modalEl.find(".team-verify-email").show();
      }
      else {
        submitCurrentForm(modalEl, form);
      }

    }, function (resp) {
      var errors = _.pairs($.parseJSON(resp.responseText));
      for (var index = 0; index < errors.length; index++) {
        var fieldKey = errors[index][0];
        var errorText = errors[index][1][0];
        if (fieldKey === "new-group-product-errors") {
          var find = form.find("#" + fieldKey);
          find.html(errorText);
          find.css('display', 'inline-block');
        }
        else {
          form.find("span[data-error-for='" + fieldKey + "']").html(errorText);
          form.find("input[name='" + fieldKey + "']").parent().addClass("error");
        }
      }
    },function () {
      modalEl.find("#new-group-product-errors").hide();
      modalEl.find("span.error-placeholder").html('');
      modalEl.find(".team-input.error").removeClass('error');
      disableButton(form);
    });
  }

  function resetAddTeamModal() {
    var modalEl = $("#addTeamPopupContent");
    modalEl.find(".team-form").show();
    modalEl.find(".team-verify-email").hide();
    modalEl.find(".team-product-checkbox").prop('checked', true);
    modalEl.find(".team-product-limit").prop('disabled', false);
    resetErrors(modalEl);
    var teamLeadCheckbox = $("#notAddTeamLeadCheckbox");
    teamLeadCheckbox.prop('checked', false);
    updateNotAddTeamLeadEmailUiState(teamLeadCheckbox);
  }

  function resetErrors(modalEl) {
    modalEl.find("#new-group-product-errors").hide();
    modalEl.find(".team-input > input").val('');
    modalEl.find("span.error-placeholder").html('');
    modalEl.find(".team-input.error").removeClass('error');
  }

  $("#notAddTeamLeadCheckbox").click(function () {
    updateNotAddTeamLeadEmailUiState(this);
  });

  function updateNotAddTeamLeadEmailUiState(button) {
    var checked = $(button).is(':checked');
    $("#notAddTeamLeadWarning").toggle(checked);
    var field = $("#teamLeadEmail");
    field.val('');
    field.prop('required', !checked);
    field.prop('disabled', checked);
    field.parent().removeClass('error');
    field.next().html('');
  }

  $(".team-product-checkbox").click(function () {
    var field = $("[name='tokens[" + $(this).data("index") + "].maxUsers']");
    field.prop('disabled', !this.checked);
    field.val('');
    field.parent().removeClass('error');
    field.next().html('');
    if (this.checked) {
      $("#new-group-product-errors").hide();
    }
  });

  $("input[name='newEmail'] + button").click(function(evt) {
    var valid = this.form.checkValidity();
    if (valid) {
      evt.preventDefault();
      addNewEmail($(this).siblings('input'));
    }
  });

  function addNewEmail(input) {
    var email = input.val().trim();
    var errorNode = input.parent().find('.alert');
    errorNode.hide();
    var form = $("form[name='addEmailTeam']");
    $.ajax({
      url: form.prop('action'),
      type: form.prop('method'),
      beforeSend: disableButton(form),
      complete: enableButton(form),
      data: {'email': email},
      dataType: 'text'
    })
    .error(function(resp) {
      errorNode.html(resp.responseText).show();
    })
    .success(function () {
      input.val('');
      addEmailToList(email);
    });
  }

  function prepareModalWindow(data) {
    var modal = $("#sendTokenContent");
    modal.find(".alert").hide();
    disableButton(modal.find("form[name='sendEmailTeam']"));
    modal.find("input[name='newEmail']").val('');
    modal.find(".fa-spinner").hide();
    $("#emails").empty();
    modal.find("form input[name='groupId']").val(data.id);
    modal.find(".team-invitation-header .team-name").text(data.name);
    modal.find("form textarea[name='content']").val(data.content);
  }

  function addEmailToList(newEmail) {
    enableButton($("#sendTokenContent").find("form[name='sendEmailTeam']"));
    var emails = $("#emails");
    if (emails.find("input[value=\"" + newEmail + "\"]").length === 0) {
      emails.append($('<li>')
        .append($('<span>').text(newEmail))
        .append($('<input type="hidden" name="emails[]"/>').val(newEmail))
        .append($('<a href="#" class="fa fa-times" aria-hidden="true"></a>')
        .click(function (evt) {
          evt.preventDefault();
          $(this).parent().remove();
        }))
        .append('</li>'));
    }
  }

  function enableButton(form) {
    form.find(':submit').removeAttr('disabled');
  }

  function disableButton(form) {
    form.find(':submit').attr('disabled','disabled');
  }
  /// Invite license users form (emailTemplate.scala.html) ///

  $("button.deactivation").click(function () {
    var modalEl = $(this).next();
    modalEl.on('hide', function(){
      modalEl.removeClass('modal');
    })
    .modal('show')
    .addClass('modal');
  });

  $('[data-clipboard-text]').each(function(idx, el) {
      new Clipboard(el, {
        text: function(trigger) {
          return $(trigger).data('clipboard-text');
        }
      })
      .on('success', function(e) {
        e.clearSelection();
      })
      .on('error', function(e) {
        console.error("Error during clipboard copying, browser likely unsupported.", e);
      });
  });
})();
